# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                               quality_check.py
#            Produces the quality check plot in the end of the pipeline
# ******************************************************************************

"""
Produces the quality check plot in the end of the pipeline

The S-PLUS field is given as the first command line argument. Configurations
are set in the config file, given as the second command line argument.

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

Assumes that all calibration steps have been run for this field

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 quality_check.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys
import numpy as np
from matplotlib import pyplot as plt
from astropy.table import Table

pipeline_path = os.path.split(__file__)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

try:
    quality_path = sys.argv[3]
except IndexError:
    quality_path = conf['quality_path'] 


################################################################################
# Get directories
suffix = ut.calibration_suffix(conf)

field_path       = os.path.join(conf['run_path'], field)
photometry_path  = os.path.join(field_path, 'Photometry')
crossmatch_path  = os.path.join(field_path, 'Crossmatch')
calibration_path = os.path.join(field_path, f'Calibration_{suffix}')

################################################################################
# Check if plot already exists

save_file = os.path.join(calibration_path, f'{field}_plots.png')

if os.path.exists(save_file):
    # Copy to quality_path
    if quality_path:
        
        suffix = ut.calibration_suffix(conf)
        save_file_quality_path = os.path.join(quality_path,
                                              f'{field}_{suffix}.png')
        
        cmd = f"cp {save_file} {save_file_quality_path}"
        os.system(cmd)

    msg = "quality plot already exists and will not be made again"
    raise FileExistsError(msg)

################################################################################
# Initiallize plots

nrow = 3
ncol = 4

fig, axs = plt.subplots(nrow, ncol, figsize=[20,15.5])
for ax in fig.axes:
    ax.tick_params(labelrotation=90, axis = 'y')

fig.suptitle(field, fontsize=14)

plt.subplots_adjust(top=0.95)

filt_colors = {'U': "#d600a6",
               'F378': "#c300bc",
               'F395': "#a700ce",
               'F410': "#8300de",
               'F430': "#3e00f3",
               'G': "#00c1ff",
               'F515': "#17ff00",
               'R': "#ff5f00",
               'F660': "#f80000",
               'I': "#cb0000",
               'F861': "#a30000",
               'Z': "#9b0000"}

################################################################################
# Plot single mode detections

def plot_single_mode_detections():

    single_master = os.path.join(photometry_path, 'single', 'master',
                                 f"{field}_master_photometry_only_single.fits")

    # Load catalog
    single_master_data = ut.load_data(single_master)

    x = single_master_data.loc[:,'X'].values
    y = single_master_data.loc[:,'Y'].values

    axs[0,0].scatter(x,y, s = 0.5, alpha = 0.1)
    axs[0,0].set_xlabel("X")
    axs[0,0].set_ylabel("Y")
    axs[0,0].set_xlim([0, 11000])
    axs[0,0].set_ylim([0, 11000])

    axs[0,0].text(100, 10500, 'Photometry master [single]')
    axs[0,0].text(100, 100, f'N_sources: {len(single_master_data)}')

try:
    plot_single_mode_detections()
except:
    pass

################################################################################
# Plot dual mode detections

def plot_dual_mode_detections():

    dual_master = os.path.join(photometry_path, 'dual', 'master',
                                 f"{field}_master_photometry_only_dual.fits")

    # Load catalog
    dual_master_data = ut.load_data(dual_master)

    x = dual_master_data.loc[:,'X'].values
    y = dual_master_data.loc[:,'Y'].values

    axs[0,1].scatter(x,y, s = 0.5, alpha = 0.1)
    axs[0,1].set_xlabel("X")
    axs[0,1].set_ylabel("Y")
    axs[0,1].set_xlim([0, 11000])
    axs[0,1].set_ylim([0, 11000])

    axs[0,1].text(100, 10500, 'Photometry master [dual]')
    axs[0,1].text(100, 100, f'N_sources: {len(dual_master_data)}')
try:
    plot_dual_mode_detections()
except:
    pass

################################################################################
# Plot psf mode detections

def plot_psf_mode_detections():

    psf_master = os.path.join(photometry_path, 'psf', 'master',
                                 f"{field}_master_photometry_only_psf.fits")

    # Load catalog
    psf_master_data = ut.load_data(psf_master)

    x = psf_master_data.loc[:,'X'].values
    y = psf_master_data.loc[:,'Y'].values

    axs[0,2].scatter(x,y, s = 0.5, alpha = 0.1)
    axs[0,2].set_xlabel("X")
    axs[0,2].set_ylabel("Y")
    axs[0,2].set_xlim([0, 11000])
    axs[0,2].set_ylim([0, 11000])

    axs[0,2].text(100, 10500, 'Photometry master [psf]')
    axs[0,2].text(100, 100, f'N_sources: {len(psf_master_data)}')
try:
    plot_psf_mode_detections()
except:
    pass


################################################################################
# Plot psf mode detections

def plot_crossmatch():

    correction = conf['extinction_correction']

    suffix = ut.calibration_suffix(conf)

    crossmatch_name = ut.crossmatch_catalog_name(field, conf)
    crossmatch_name = crossmatch_name.replace(".fits",
                                              f"_ebvcorr_{correction}.fits")

    crossmatch = os.path.join(crossmatch_path, crossmatch_name)

    # Load catalog
    crossmatch_data = ut.load_data(crossmatch)

    x = crossmatch_data.loc[:,'X'].values
    y = crossmatch_data.loc[:,'Y'].values

    axs[0,3].scatter(x,y, s = 2, alpha = 1)
    axs[0,3].set_xlabel("X")
    axs[0,3].set_ylabel("Y")
    axs[0,3].set_xlim([0, 11000])
    axs[0,3].set_ylim([0, 11000])

    txt = (f"Crossmatch [ {conf['calibration_photometry']} | "
           f"{' '.join(conf['reference_catalog'])} | "
           f"{conf['extinction_correction']} ]")
    axs[0,3].text(100, 10500, txt)
    axs[0,3].text(100, 100, f'N_sources: {len(crossmatch_data)}')

try:
    plot_crossmatch()
except:
    pass


################################################################################
# Plot FWHM det

def plot_fwhm():

    mag_cut = [10,25]

    detection = os.path.join(photometry_path, 'dual', 'detection',
                             f'sex_{field}_detection_catalog.fits')

    # Load catalog
    detection_data = Table.read(detection)

    select, medianFWHM = ut.star_selector(catalog=detection_data,
                                          s2ncut=conf['apercorr_s2ncut'],
                                          starcut=conf['apercorr_starcut'],
                                          mag_partition=2,
                                          verbose=False)

    sexconf = os.path.join(photometry_path, 'dual', 'detection',
                             f'{field}_detection.sex')

    fwhm_config = ut.get_sexconf_fwhm(sexconf=sexconf)
    fwhm_estimated = medianFWHM * 3600

    ax = axs[1,0]
    ax.scatter(select['MAG_AUTO'], select['FWHM_WORLD'] * 3600,
               c="#2266FF", s=10, alpha=0.3, zorder=2)

    ax.scatter(detection_data['MAG_AUTO'],
                     detection_data['FWHM_WORLD'] * 3600,
                     c="#AAAAAA", s=10, alpha=0.1)

    ax.plot([mag_cut[0], mag_cut[1]], [fwhm_config, fwhm_config],
                color="#FF6622", zorder=1,
                label=r"FWHM$_{\mathrm{config}}$" + f": {fwhm_config:.3f}")

    ax.plot([mag_cut[0], mag_cut[1]], [fwhm_estimated, fwhm_estimated],
                color="#22AA66", zorder=1,
                label=r"FWHM$_{\mathrm{det_image}}$" + f": {fwhm_estimated:.3f}")

    ax.set_xlabel("MAG_DET_AUTO")
    ax.set_ylabel("FWHM [arcsec]")
    ax.set_xlim(mag_cut)
    ax.set_ylim([0, 10])

    ax.legend()

try:
    plot_fwhm()
except:
    pass

################################################################################
# Plot Growth curves (dual)

def plot_growth_curves():

    filters = conf['filters']
    ax = axs[1,1]

    aperture = conf['apercorr_diameter']
    max_aperture = conf['apercorr_max_aperture']
    maxy = 0.15
    miny = -0.5

    # max aperture
    ax.plot([max_aperture, max_aperture], [miny, maxy], '-', color='purple',
             linewidth = 0.5)

    ax.text(max_aperture, 0.05, f'{max_aperture:.3f}\ndiameter',
            color='purple', fontsize = 8)

    # base aperture
    ax.plot([aperture, aperture], [miny, maxy], '-', color='blue',
             linewidth = 0.5)

    ax.text(aperture, 0.05, f'{aperture:.3f}\ndiameter',
            color='blue', fontsize = 8)

    # region around zero
    ax.plot([0, 2*max_aperture], [0, 0], color='black', linewidth = 0.5)
    ax.fill_between([0, 2*max_aperture], [-1e-2, -1e-2], [1e-2, 1e-2],
                     color='black', alpha=0.1)

    for filt in filters:
        print(filt)
        color = filt_colors[filt]

        sexconf = os.path.join(photometry_path, 'dual', 'sexconf',
                             f'{field}_{filt}_dual.sex')

        apertures_list = ut.get_apertures_from_sexconf(sexconf)

        if 'correction_xy' in conf['run_steps']:
            catalog_dir = 'xy_correction'
            suffix = '_xycorr'
        else:
            catalog_dir = 'catalogs'
            suffix = ''

        catalog_file = os.path.join(photometry_path, 'dual', catalog_dir,
                                    f'sex_{field}_{filt}_dual{suffix}.fits')

        result, medianFWHM, starcount = ut.growth_curve(catalog=catalog_file,
                                               s2ncut=conf['apercorr_s2ncut'],
                                               starcut=conf['apercorr_starcut'],
                                               verbose=False)

        minmag2, minmag, medianmag, maxmag, maxmag2 = [x for x in result]

        diameter = [(a + b) / 2 for a, b in zip(apertures_list[:],
                                                apertures_list[1:])]

        # medians
        ax.plot(diameter, medianmag, color=color, linewidth = 0.5,
                label=f'{filt} ({starcount})')

        # CL68 & CL95
        ax.fill_between(diameter, minmag, maxmag, color=color, alpha=0.1)

    # Plot parameters
    ax.set_ylim([miny, maxy])
    ax.set_xlim([0, 1.2*max_aperture])

    # Labels
    ax.legend(loc=4, ncol = 2)
    ax.set_xlabel("Aperture Diameter (pix)")
    ax.set_ylabel("$m_{k+1} - m_{k}$")

try:
    plot_growth_curves()
except:
    pass


################################################################################
# Crossmatch dual and psf photometry

def crossmatch_dual_psf():

    dual_catalog = os.path.join(photometry_path, 'dual', 'master',
                                f'{field}_master_photometry_only_dual.fits')

    psf_catalog = os.path.join(photometry_path, 'psf', 'master',
                                f'{field}_master_photometry_only_psf.fits')

    save_file = os.path.join(crossmatch_path,
                             f'crossmatch_master_dual_psf.fits')

    if not os.path.exists(dual_catalog):
        raise ValueError(f"file {dual_catalog} does not exist.")

    if not os.path.exists(psf_catalog):
        raise ValueError(f"file {psf_catalog} does not exist.")

    if not os.path.exists(save_file):
        cmd  = f'java -jar {conf["path_to_stilts"]} tmatch2 '
        cmd += f'in1={dual_catalog} ifmt1=fits values1="RAJ2000 DEJ2000" '
        cmd += f'in2={psf_catalog} ifmt2=fits values2="RAJ2000 DEJ2000" '
        cmd += f'out={save_file} ofmt=fits '
        cmd += f'find=best join=1and2 matcher=sky params=1'

        print(cmd)
        os.system(cmd)


try:
    crossmatch_dual_psf()
except:
    pass


################################################################################
# Plot dual - psf photometry

def plot_dual_vs_psf():

    shift = [6,5,4,3,2,1,0,-1,-2,-3,-4,-5]

    catalog = os.path.join(crossmatch_path,
                           f'crossmatch_master_dual_psf.fits')

    catalog_data = ut.load_data(catalog)

    filters = conf['filters']
    ax = axs[1,2]
    ax.set_xlim([8,22])
    ax.set_ylim([-6,7])

    ax.set_xlabel("mag dual (PStotal)")
    ax.set_ylabel("mag dual - mag psf")

    ax.set_yticks([-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6])
    ax.set_yticklabels(['', '', '', '', -1, 0, 1, '', '', '', '', ''],
                       minor=False)

    # Get zps
    zp_file = os.path.join(calibration_path, f'{field}_final.zp')
    zp_data = ut.zp_read(zp_file)

    for i in range(len(filters)):
        filt = filters[i]
        color = filt_colors[filt]

        zp_filt = zp_data[f'SPLUS_{filt}']-20

        ax.text(8.5, shift[i]+0.1, filt, color = color)

        mag_dual = catalog_data.loc[:,f'SPLUS_{filt}_1'].values + zp_filt
        mag_psf = catalog_data.loc[:,f'SPLUS_{filt}_2'].values + zp_filt

        delta = mag_dual - mag_psf
        f = np.abs(delta) < 1

        ax.scatter(mag_dual[f], delta[f]+shift[i], s = 2, alpha = 0.1,
                    color = color, zorder = 1)

        ax.plot([8,22],[shift[i],shift[i]], '--', color = 'black', alpha = 0.5,
                zorder = 0, linewidth = 0.5)

try:
    plot_dual_vs_psf()
except:
    pass

################################################################################
# Plot internal SED fitting


def plot_internal_sed_fitting():

    filters = conf['filters']
    shift = [6, 5, 4, 3, 2, 1, 0, -1, -2, -3, -4, -5]
    mag_cut = conf['zp_fitting_mag_cut']

    sed_fit_file = os.path.join(calibration_path, 'internal',
                                f"{field}_mag_int.cat")

    cat_data = ut.load_data(sed_fit_file)

    ax = axs[1,3]
    ax.set_xlim([12,22])
    ax.set_ylim([-6,7])

    ax.set_xlabel("model mag")
    ax.set_ylabel("model mag - splus mag")

    ax.set_yticks([-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5, 6])
    ax.set_yticklabels(['', '', '', '', -0.5, 0, 0.5, '', '', '', '', ''],
                       minor=False)

    for i in range(len(filters)):
        filt = filters[i]
        color = filt_colors[filt]

        # Remove 99
        data_selection = np.abs(cat_data.loc[:, f'SPLUS_{filt}'] < 50)
        cat_data = cat_data[data_selection]

        ax.text(12.3, shift[i]+0.1, filt, color = color)

        ##############
        # Prepare data

        x = cat_data.loc[:, f'SPLUS_{filt}_mod']
        y = 2*(x - (cat_data.loc[:, f'SPLUS_{filt}']))

        dwarfs = cat_data.loc[:, 'logg'].values > 3
        selection = (x >= mag_cut[0]) & (x <= mag_cut[1]) & dwarfs

        ############################
        # Plot

        ax.scatter(x[selection], y[selection] + shift[i],
                   c=color, s=5, alpha=0.5, zorder=2)

        ax.scatter(x[~selection], y[~selection] + shift[i],
                   c="#666666", s=2, alpha=0.1, zorder=1)

        ax.plot([8,22],[shift[i],shift[i]], '--', color = 'black', alpha = 0.5,
                zorder = 0, linewidth = 0.5)

        ax.plot([mag_cut[0], mag_cut[0]], [-6,7],
                 color='#0066FF', linestyle='-', linewidth=0.5, zorder=6)

        ax.plot([mag_cut[1], mag_cut[1]], [-6,7],
                 color='#0066FF', linestyle='-', linewidth=0.5, zorder=6)


try:
    plot_internal_sed_fitting()
except:
    pass


################################################################################
# Plot internal SED fitting

def plot_class_star():

    detection = os.path.join(photometry_path, 'dual', 'detection',
                             f'sex_{field}_detection_catalog.fits')

    sexcat = Table.read(detection)
    starcut = conf['apercorr_starcut']
    s2ncut = conf['apercorr_s2ncut']
    mag_cut = [10,25]

    select, medianFWHM = ut.star_selector(catalog=sexcat,
                                          s2ncut=s2ncut,
                                          starcut=starcut,
                                          mag_partition=2,
                                          verbose=False)

    ax = axs[2,0]

    ax.scatter(sexcat['MAG_AUTO'], sexcat['CLASS_STAR'],
                   c="#AAAAAA", s=10, alpha=0.1)

    # Plot selection
    ax.scatter(select['MAG_AUTO'], select['CLASS_STAR'],
                   c="#2266FF", s=10, alpha=0.3,
                   label=f"s2ncut: {s2ncut}\n& starcut: {starcut}")

    # Plot cut line
    ax.hlines(starcut, mag_cut[0], mag_cut[1],
                  colors="#2266FF", zorder=1)

    # Plot grid lines
    ax.hlines(np.arange(-0.1, 1.1, 0.1), mag_cut[0], mag_cut[1],
                  colors="#EEEEEE", zorder=-1)

    ax.legend(loc=1)
    ax.set_xlim(mag_cut)
    ax.set_ylim([-0.1, 1.1])
    ax.set_ylabel('CLASS_STAR')
    ax.set_xlabel('MAG_AUTO (detection)')
    ax.minorticks_on()

try:
    plot_class_star()
except:
    pass

################################################################################
# Plot color-color 1

def plot_color_color_1():

    ax = axs[2,1]
    ax.set_xlim([0.75,3])
    ax.set_ylim([0.5,4])

    ax.set_xlabel("U - G")
    ax.set_ylabel("F378 - F660")

    stellar_locus_reference_file = conf['stellar_locus_reference']
    sl_ref = ut.load_data(stellar_locus_reference_file)

    # Plot reference stellar locus
    x_ref = sl_ref.loc[:,'SPLUS_U'].values - sl_ref.loc[:,'SPLUS_G'].values
    y_ref = sl_ref.loc[:,'SPLUS_F378'].values - sl_ref.loc[:,'SPLUS_F660'].values

    ax.scatter(x_ref, y_ref, color = "#AAAAAA", s = 2, alpha = 0.05, zorder = -1)

    # Load field data (calibrated and EB-V corrected)
    catalog_file = os.path.join(calibration_path, 'internal',
                                f'{field}_mag_int.cat')

    cat_data = ut.load_data(catalog_file)

    x1 = cat_data.loc[:,'SPLUS_U'].values
    x2 = cat_data.loc[:,'SPLUS_G'].values
    y1 = cat_data.loc[:,'SPLUS_F378'].values
    y2 = cat_data.loc[:,'SPLUS_F660'].values

    f = (x1 < 19) & (x2 < 19) & (y1 < 19) & (y2 < 19)

    x_cat = x1[f] - x2[f]
    y_cat = y1[f] - y2[f]

    ax.scatter(x_cat, y_cat, color = "#FF5F00", s = 5, alpha = 0.5, zorder = 1)

try:
    plot_color_color_1()
except:
    pass

################################################################################
# Plot color-color 2

def plot_color_color_2():

    ax = axs[2,2]
    ax.set_xlim([0.5,3.5])
    ax.set_ylim([0,3.5])

    ax.set_xlabel("F395 - R")
    ax.set_ylabel("F410 - F861")

    stellar_locus_reference_file = conf['stellar_locus_reference']
    sl_ref = ut.load_data(stellar_locus_reference_file)

    # Plot reference stellar locus
    x_ref = sl_ref.loc[:,'SPLUS_F395'].values - sl_ref.loc[:,'SPLUS_R'].values
    y_ref = sl_ref.loc[:,'SPLUS_F410'].values - sl_ref.loc[:,'SPLUS_F861'].values

    ax.scatter(x_ref, y_ref, color = "#AAAAAA", s = 2, alpha = 0.05, zorder = -1)

    # Load field data (calibrated and EB-V corrected)
    catalog_file = os.path.join(calibration_path, 'internal',
                                f'{field}_mag_int.cat')

    cat_data = ut.load_data(catalog_file)

    x1 = cat_data.loc[:,'SPLUS_F395'].values
    x2 = cat_data.loc[:,'SPLUS_R'].values
    y1 = cat_data.loc[:,'SPLUS_F410'].values
    y2 = cat_data.loc[:,'SPLUS_F861'].values

    f = (x1 < 19) & (x2 < 19) & (y1 < 19) & (y2 < 19)

    x_cat = x1[f] - x2[f]
    y_cat = y1[f] - y2[f]

    ax.scatter(x_cat, y_cat, color = "#FF5F00", s = 5, alpha = 0.5, zorder = 1)

try:
    plot_color_color_2()
except:
    pass


################################################################################
# Plot color-color 3

def plot_color_color_3():

    ax = axs[2,3]
    ax.set_xlim([0.3,2.5])
    ax.set_ylim([0,2])

    ax.set_xlabel("F430 - I")
    ax.set_ylabel("F515 - Z")

    stellar_locus_reference_file = conf['stellar_locus_reference']
    sl_ref = ut.load_data(stellar_locus_reference_file)

    # Plot reference stellar locus
    x_ref = sl_ref.loc[:,'SPLUS_F430'].values - sl_ref.loc[:,'SPLUS_I'].values
    y_ref = sl_ref.loc[:,'SPLUS_F515'].values - sl_ref.loc[:,'SPLUS_Z'].values

    ax.scatter(x_ref, y_ref, color = "#AAAAAA", s = 2, alpha = 0.05, zorder = -1)

    # Load field data (calibrated and EB-V corrected)
    catalog_file = os.path.join(calibration_path, 'internal',
                                f'{field}_mag_int.cat')

    cat_data = ut.load_data(catalog_file)

    x1 = cat_data.loc[:,'SPLUS_F430'].values
    x2 = cat_data.loc[:,'SPLUS_I'].values
    y1 = cat_data.loc[:,'SPLUS_F515'].values
    y2 = cat_data.loc[:,'SPLUS_Z'].values

    f = (x1 < 19) & (x2 < 19) & (y1 < 19) & (y2 < 19)

    x_cat = x1[f] - x2[f]
    y_cat = y1[f] - y2[f]

    ax.scatter(x_cat, y_cat, color = "#FF5F00", s = 5, alpha = 0.5, zorder = 1)

try:
    plot_color_color_3()
except:
    pass

plt.tight_layout()

plt.savefig(save_file)

if quality_path:
    suffix = ut.calibration_suffix(conf)
    extinction = conf['extinction_correction']
    save_file_quality_path = os.path.join(quality_path,
                                         f'{field}_{suffix}.png')

    plt.savefig(save_file_quality_path)


plt.clf()
plt.close()



# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                               photometry_psf.py
#                       psf photometry using dophot_splus
# ******************************************************************************

"""
Runs dophot_splus for one S-PLUS field and a list of filters.

The S-PLUS field is given as the first command line argument.

The set of filters, location of S-PLUS images, default dophot configuration,
and the output path must be set in the configuration file
given as the second command line argument

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------

create_photometry_path()
copy_images_to_dophot_path()
generate_tuneup_files()
run_dophot()
delete_images_from_dophot_path()

--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 photometry_psf.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys

steps_path = os.path.split(__file__)[0]
pipeline_path = os.path.split(steps_path)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

################################################################################
# Get directories

field_path      = os.path.join(conf['run_path'], field)
photometry_path = os.path.join(field_path, 'Photometry')
psf_path        = os.path.join(photometry_path, 'psf')
dophot_path     = os.path.join(psf_path, 'dophot')
catalogs_path   = os.path.join(psf_path, 'catalogs')

images_path     = os.path.join(field_path, 'Images')

log_path        = os.path.join(photometry_path, 'logs')

################################################################################
# Create Photometry directory

ut.makedir(field_path)
ut.makedir(photometry_path)
ut.makedir(log_path)

################################################################################
# Initiate log file

log_file_name = os.path.join(log_path, 'photometry_psf.log')
log_file_name = ut.gen_logfile_name(log_file_name)
log_file = os.path.join(photometry_path, log_file_name)

with open(log_file, "w") as log:
    log.write("")

################################################################################
# Log configuration

ut.printlog("PSF photometry parameters:", log_file)

psf_params = ['run_path', 'filters', 'path_to_images',
              'path_to_dophot', 'dophot_config']

for param in psf_params:
    try:
        ut.printlog(f"{param}: {conf[param]}", log_file)
    except KeyError:
        ut.printlog(f"{param}: NONE", log_file)

################################################################################
# Begin script

# ***************************************************
#    If photometry path does not exist -> create
# ***************************************************


def create_photometry_path():

    print("")
    ut.printlog(('********** '
                 'Generating PSF Photometry paths '
                 '**********'),
                 log_file)
    print("")

    # Create single mode photometry path
    ut.makedir(psf_path)

    ut.makedir(dophot_path)

    ut.makedir(catalogs_path)


create_photometry_path()

# ***************************************************
#    Copy images to dophot path
# ***************************************************


def copy_images_to_dophot_path():

    """
    Copying images to dophot path
    """

    print("")
    ut.printlog(('********** '
                 'Copying images to dophot path '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        # Get location of image file
        image_name      = f"{field}_{filt}_swp.fits"
        image_file_from = os.path.join(images_path, image_name)
        image_file_to   = os.path.join(dophot_path, image_name)

        if not os.path.exists(image_file_to):

            # Copy image file from source location
            cmd = f"cp {image_file_from} {image_file_to}"
            ut.printlog("$ "+cmd, log_file)
            os.system(cmd)

        else:
            ut.printlog(f"Image {image_name} already in dophot directory.",
                        log_file)


copy_images_to_dophot_path()

# ***************************************************
#    Generate dophot-splus tuneup files
# ***************************************************


def generate_tuneup_files():

    """
    Generate psf tuneup files for all filters
    """

    print("")
    ut.printlog(('********** '
                 'Generating PSF Photometry tuneup files '
                 '**********'),
                 log_file)
    print("")

    # Generate tuneup for each filter
    for filt in conf['filters']:

        # Generate location of config file
        conf_file_name = f"{field}_{filt}_swp.tuneup"
        save_file = os.path.join(dophot_path, conf_file_name)

        # Generate location of output catalog file
        catalog_name = f"{field}_{filt}_swp.sum"
        catalog_file = os.path.join(dophot_path, catalog_name)

        # Get location of image file
        image_name = f"{field}_{filt}_swp.fits"
        image_file = os.path.join(images_path, image_name)

        if not os.path.exists(save_file):

            ut.get_dophot_config(image_in    = image_file,
                                 objects_out = catalog_file,
                                 config_file = save_file,
                          apercorr_max_aperture = conf['apercorr_max_aperture'])

            ut.printlog(("Generated PSF tuneup file for field "
                         f"{field}, filter {filt}"), log_file)

        else:
            ut.printlog(("PSF tuneup file already obtained for field "
                         f"{field}, filter {filt}"), log_file)

    # Copy paramdefault file to dophot directory
    paramdefault_from = conf['dophot_config']
    paramdefault_to   = os.path.join(dophot_path, 'paramdefault')

    cmd = f'cp {paramdefault_from} {paramdefault_to}'
    ut.printlog("$ "+cmd, log_file)
    os.system(cmd)


generate_tuneup_files()

# ***************************************************
#    Run dophot-splus
# ***************************************************


def run_dophot():

    """
    Run dophot-splus to extract PSF photometry from all filters
    """

    print("")
    ut.printlog(('********** '
                 'Running dophot-splus '
                 '**********'),
                 log_file)
    print("")

    # Get current directory (to return after running dophot)
    cwd = os.getcwd()

    # Change working directory to dophot directory
    os.chdir(dophot_path)

    for filt in conf['filters']:

        catalog_name = f"{field}_{filt}_swp.sum"
        catalog_file = os.path.join(dophot_path, catalog_name)

        if not os.path.exists(catalog_file):

            # Get location of tuneup
            tuneup_file_name = f"{field}_{filt}_swp.tuneup"

            # Run dophot
            dophot_location = os.path.join(conf['path_to_dophot'],
                                           'dophot_splus')

            cmd = f"{dophot_location} {tuneup_file_name}"
            ut.printlog("$ "+cmd, log_file)
            os.system(cmd)

        else:
            ut.printlog(("PSF photometry already obtained for field "
                         f"{field}, filter {filt}"), log_file)

    # Go back to current working directory
    os.chdir(cwd)


run_dophot()


# ***************************************************
#    Format output catalogs
# ***************************************************


def format_dophot_catalogs():

    """
    Formats dophot-splus .sum files into ascii catalogs
    """

    print("")
    ut.printlog(('********** '
                 'Formating dophot catalogs '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        dophot_catalog_name = f"{field}_{filt}_swp.sum"
        dophot_catalog_file = os.path.join(dophot_path, dophot_catalog_name)

        formated_catalog_name = f"{field}_{filt}_psf.cat"
        formated_catalog_file = os.path.join(catalogs_path,
                                             formated_catalog_name)

        image_name = f"{field}_{filt}_swp.fz"
        image_file = os.path.join(images_path, image_name)

        if not os.path.exists(formated_catalog_file):

            ut.format_dophot_catalog(catalog_in  = dophot_catalog_file,
                                     catalog_out = formated_catalog_file,
                                     image       = image_file,
                                     filt        = filt,
                                     field       = field,
                                     drname      = conf['data_release_name'])

        else:
            ut.printlog(("Dophot catalog already formated for field "
                         f"{field}, filter {filt}"), log_file)


format_dophot_catalogs()

# ***************************************************
#    Delete images from dophot path
# ***************************************************


def delete_images_from_dophot_path():

    """
    Deletes images from dophot path
    """

    print("")
    ut.printlog(('********** '
                 'Deleting images from dophot path '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        # Get location of image file
        image_name = f"{field}_{filt}_swp.fits"
        image_file = os.path.join(dophot_path, image_name)

        if os.path.exists(image_file):

            # Copy image file from source location
            cmd = f"rm {image_file}"
            ut.printlog("$ "+cmd, log_file)
            os.system(cmd)

        else:
            ut.printlog((f"Image {image_name} already deleted from "
                          "dophot directory."), log_file)


delete_images_from_dophot_path()

ut.printlog("COMPLETED PSF PHOTOMETRY", log_file)


# ***************************************************
#    PSF diagnostic plots
# ***************************************************

def psf_diagnostic_plots():

    """
    Makes psf diagnostic plots of the resulting catalogs
    """

    print("")
    ut.printlog(('********** '
                 'Making diagnostic plots for PSF photometry'
                 '**********'),
                log_file)
    print("")

    for filt in conf['filters']:

        # Generate location of output catalog file
        catalog_name = f"{field}_{filt}_psf.cat"
        catalog_file = os.path.join(catalogs_path, catalog_name)

        save_name = f"{field}_{filt}_psf.png"
        save_file = os.path.join(catalogs_path, save_name)

        if not os.path.exists(save_file):

            ut.plot_dophot_diagnostic(catalog   = catalog_file,
                                      save_file = save_file,
                                      filt      = filt)

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(("Diagnostic plots already made for field "
                         f"{field}, filter {filt}"), log_file)


psf_diagnostic_plots()

ut.printlog("COMPLETED PSF PHOTOMETRY", log_file)


# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                             calibration_stloc.py
#            Runs the stellar locus calibration step of the pipeline
# ******************************************************************************

"""
Obtains the stellar locus calibration zero-points by comparing the stellar
locus (with a single not calibrated magnitude) to a reference stellar locus

The S-PLUS field is given as the first command line argument. Configurations
are set in the config file, given as the second command line argument.

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------

get_and_plot_stellar_locus_zps()
apply_stellar_locus_zero_points()
combine_external_and_stlocus_zps()

--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

Assumes that calibration_external.py has already been run for this field.

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 calibration_stloc.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys

steps_path = os.path.split(__file__)[0]
pipeline_path = os.path.split(steps_path)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

################################################################################
# Get directories

field_path = os.path.join(conf['run_path'], field)

suffix = ut.calibration_suffix(conf)
calibration_path  = os.path.join(field_path, f'Calibration_{suffix}')

external_cal_path = os.path.join(calibration_path, 'external')
external_plot_path = os.path.join(external_cal_path, 'plots')

stloc_cal_path = os.path.join(calibration_path, 'stlocus')
stloc_plot_path = os.path.join(stloc_cal_path, 'plots')

log_path = os.path.join(calibration_path, 'logs')


################################################################################
# Initiate log file

ut.makedir(stloc_cal_path)
ut.makedir(stloc_plot_path)

log_file_name = os.path.join(log_path, 'calibration_stloc.log')
log_file_name = ut.gen_logfile_name(log_file_name)
log_file = os.path.join(calibration_path, log_file_name)

with open(log_file, "w") as log:
    log.write("")


################################################################################
# Begin script

def get_and_plot_stellar_locus_zps():

    """
    Obtain zero-points using the stellar locus technique. Also saves the plots.
    """

    print("")
    ut.printlog(('********** '
                 'Getting stellar locus ZPs '
                 '**********'),
                 log_file)
    print("")

    catalog_name = f"{field}_mag_ext.cat"
    catalog_file = os.path.join(external_cal_path, catalog_name)

    save_name = f"{field}_stlocus_tmp.zp"
    save_file = os.path.join(stloc_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.zp_estimate_stlocus(catalog   = catalog_file,
                               save_file = save_file,
                               filts_to_get_zp = conf['stellar_locus_fit'],
                               stlocus_ref_cat = conf['stellar_locus_reference'],
                               filts_color_ref = conf['stellar_locus_color_ref'],
                               filt_ref = conf['stellar_locus_filt_ref'],
                               color_range = conf['stellar_locus_color_range'],
                               nbins = conf['stellar_locus_N_bins'],
                               plot_path = stloc_plot_path,
                               field = field)

        ut.printlog(f"Zero-points saved in {save_file}", log_file)

    else:
        ut.printlog((f"Stellar locus ZPs already obtained and saved in "
                     f"{save_name}"), log_file)


get_and_plot_stellar_locus_zps()


# ***************************************************
#    Add offset to S-PLUS reference calibration
# ***************************************************

def add_offset_to_splus_reference_calibration():
    """
    Adds offset to S-PLUS reference calibration
    """

    if conf['offset_include_after'].lower() == "stloc":
        print("")
        ut.printlog(('********** '
                     'Adding offset to S-PLUS reference calibration '
                     '**********'),
                    log_file)
        print("")

    stloc_zp_name = f"{field}_stlocus_tmp.zp"
    stloc_zp_file = os.path.join(stloc_cal_path, stloc_zp_name)

    save_name = f"{field}_stlocus.zp"
    save_file = os.path.join(stloc_cal_path, save_name)

    if not os.path.exists(save_file):

        if conf['offset_include_after'].lower() == "stloc":
            offset_zp_name = f'offset_refcalib.zp'
            offset_zp_file = os.path.join(stloc_cal_path, offset_zp_name)

            cmd = f"cp {conf['offset_to_splus_refcalib']} {offset_zp_file}"
            ut.printlog(cmd, log_file)
            os.system(cmd)

            zp_file_list = [stloc_zp_file, offset_zp_file]

            filters = []
            for filt in conf['filters']:
                filters.append(f"SPLUS_{filt}")

            # Combine zero-points
            ut.zp_add(zp_file_list=zp_file_list,
                      save_file=save_file,
                      filters=filters,
                      inst_zp=0)

        else:
            cmd = f"cp {stloc_zp_file} {save_file}"
            os.system(cmd)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"File {save_name} already exists", log_file)


add_offset_to_splus_reference_calibration()


# ***************************************************
#    Apply Zero Points
# ***************************************************

def apply_stellar_locus_zero_points():

    """
    Applies stellar locus zero-points to mag_ext catalog
    """

    print("")
    ut.printlog(('********** '
                 'Applying stellar locus zero-points to mag_ext catalog '
                 '**********'),
                 log_file)
    print("")

    catalog_name = f"{field}_mag_ext.cat"
    catalog_file = os.path.join(external_cal_path, catalog_name)

    zp_name = f"{field}_stlocus.zp"
    zp_file = os.path.join(stloc_cal_path, zp_name)

    save_name = f"{field}_mag_stlocus.cat"
    save_file = os.path.join(stloc_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.zp_apply(catalog   = catalog_file,
                    save_file = save_file,
                    zp_file   = zp_file)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


apply_stellar_locus_zero_points()


# ***************************************************
#    Combine external and st_locus zps
# ***************************************************


def combine_external_and_stlocus_zps():

    """
    Combines the external and stlocus .zp files into a single .zp file
    """

    print("")
    ut.printlog(('********** '
                 'Combining external and stlocus .zp files '
                 '**********'),
                 log_file)
    print("")

    ext_zp_name = f"{field}_external.zp"
    ext_zp_file = os.path.join(external_cal_path, ext_zp_name)

    stlocus_zp_name = f"{field}_stlocus.zp"
    stlocus_zp_file = os.path.join(stloc_cal_path, stlocus_zp_name)

    save_name = f"{field}_external_and_stlocus.zp"
    save_file = os.path.join(stloc_cal_path, save_name)

    filters = []
    for filt in conf['filters']:
        filters.append(f"SPLUS_{filt}")

    if not os.path.exists(save_file):

        ut.zp_add(zp_file_list = [ext_zp_file, stlocus_zp_file],
                  save_file = save_file,
                  filters   = filters)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


combine_external_and_stlocus_zps()



import setuptools

setuptools.setup(
   name='spluscalib',
   version='0.2.7',
   description='S-PLUS calibration pipeline',
   author='Felipe Almeida Fernandes',
   author_email='felipefer42@gmail.com',
   packages=setuptools.find_packages(),
   install_requires=['astropy', 'astroquery', 'sfdmap', 'numpy',
                     'pandas', 'scipy', 'sklearn', 'matplotlib',
                     'shapely', 'Pillow'],
   url='https://github.com/felipefer42/spluscalib',
   python_requires='>3.6.0',
   include_package_data=True
)

# python setup.py sdist
# python setup.py bdist_wheel

# twine upload --repository-url https://test.pypi.org/legacy/ dist/spluscalib-0.1.10*
# pip3 install --index-url https://test.pypi.org/simple/ spluscalib --user --version==0.1.10

# twine upload dist/spluscalib-0.2*
# pip3 install spluslcalib version==0.2.3



# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                             calibration_catalog.py
#            Applies the final zero-point to the photometry catalogs
# ******************************************************************************

"""
Generates photometric calibrated catalogs from the final zero-points and the
photometry output tables

The S-PLUS field is given as the first command line argument. Configurations
are set in the config file, given as the second command line argument.

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------
copy_splus_psf_inst_catalog()
apply_final_zero_points_to_psf()

--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

Assumes that at least calibration_finalzp.py has already been run for this field

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 calibration_catalog.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys

steps_path = os.path.split(__file__)[0]
pipeline_path = os.path.split(steps_path)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

################################################################################
# Get directories

field_path       = os.path.join(conf['run_path'], field)
crossmatch_path  = os.path.join(field_path, 'Crossmatch')

suffix = ut.calibration_suffix(conf)
calibration_path  = os.path.join(field_path, f'Calibration_{suffix}')

catalogs_path = os.path.join(calibration_path, field)
catalogs_path_psf    = os.path.join(catalogs_path, 'psf')
catalogs_path_single = os.path.join(catalogs_path, 'single')
catalogs_path_dual   = os.path.join(catalogs_path, 'dual')

photometry_path = os.path.join(field_path, 'Photometry')

psf_path              = os.path.join(photometry_path, 'psf')
psf_catalog_path      = os.path.join(psf_path, 'catalogs')
psf_xycorrection_path = os.path.join(psf_path, 'xy_correction')
psf_master_path       = os.path.join(psf_path, 'master')
psf_id_path  = os.path.join(psf_path, 'id')

single_path         = os.path.join(photometry_path, 'single')
single_catalog_path = os.path.join(single_path, 'aper_correction')
single_master_path  = os.path.join(single_path, 'master')
single_id_path  = os.path.join(single_path, 'id')

dual_path         = os.path.join(photometry_path, 'dual')
dual_catalog_path = os.path.join(dual_path, 'aper_correction')
dual_master_path  = os.path.join(dual_path, 'master')
dual_detection_path = os.path.join(dual_path, 'detection')
dual_id_path  = os.path.join(dual_path, 'id')

log_path = os.path.join(calibration_path, 'logs')

################################################################################
# Initiate log file


ut.makedir(catalogs_path)
ut.makedir(log_path)

# Check for photometry folders
has_photometry_single = "photometry_single" in conf["run_steps"]
has_photometry_dual   = "photometry_dual" in conf["run_steps"]
has_photometry_psf    = "photometry_psf" in conf["run_steps"]

has_photometry_single &= os.path.exists(single_catalog_path)
has_photometry_dual   &= os.path.exists(dual_catalog_path)
has_photometry_psf    &= os.path.exists(psf_catalog_path)

if has_photometry_single:
    ut.makedir(catalogs_path_single)

if has_photometry_dual:
    ut.makedir(catalogs_path_dual)

if has_photometry_psf:
    ut.makedir(catalogs_path_psf)

log_file_name = os.path.join(log_path, 'catalog.log')
log_file_name = ut.gen_logfile_name(log_file_name)
log_file = os.path.join(calibration_path, log_file_name)

with open(log_file, "w") as log:
    log.write("")

################################################################################
# Begin script

# ***************************************************
#    Make single mode filter catalogs
# ***************************************************


def generate_singlemode_filter_catalogues():

    """
    Generates single mode calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating single mode calibrated filter catalogues '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        catalog_name = f'sex_{field}_{filt}_single_withIDs.fits'
        catalog_file = os.path.join(single_id_path, catalog_name)

        zp_name = f'{field}_final.zp'
        zp_file = os.path.join(calibration_path, zp_name)

        save_name = f'{field}_{filt}_single.fits'
        save_file = os.path.join(catalogs_path_single, save_name)

        if not os.path.exists(save_file):

            ut.sexcatalog_apply_calibration(catalog_file = catalog_file,
                                            zp_file      = zp_file,
                                            save_file    = save_file,
                                            filter_name  = filt,
                                            field        = field,
                                            sex_mag_zp   = conf['inst_zp'],
                                            mode         = 'single',
                                      calibration_flag=conf['calibration_flag'],
                                      calibration_name=conf['calibration_name'],
                              extinction_maps_path=conf['extinction_maps_path'],
                            extinction_correction=conf['extinction_correction'])

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_single:
    generate_singlemode_filter_catalogues()


# ***************************************************
#    Make dual mode filter catalogs
# ***************************************************

def generate_dualmode_filter_catalogues():

    """
    Generates dual mode calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating dual mode calibrated filter catalogues '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        catalog_name = f'sex_{field}_{filt}_dual_withIDs.fits'
        catalog_file = os.path.join(dual_id_path, catalog_name)

        zp_name = f'{field}_final.zp'
        zp_file = os.path.join(calibration_path, zp_name)

        save_name = f'{field}_{filt}_dual.fits'
        save_file = os.path.join(catalogs_path_dual, save_name)

        if not os.path.exists(save_file):

            ut.sexcatalog_apply_calibration(catalog_file = catalog_file,
                                            zp_file      = zp_file,
                                            save_file    = save_file,
                                            filter_name  = filt,
                                            field        = field,
                                            sex_mag_zp   = conf['inst_zp'],
                                            mode         = 'dual',
                                            drname=conf["data_release_name"],
                                      calibration_flag=conf['calibration_flag'],
                                      calibration_name=conf['calibration_name'])

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_dual:
    generate_dualmode_filter_catalogues()


# ***************************************************
#    Make dual mode filter catalogs
# ***************************************************

def generate_detection_catalogue():

    """
    Generates dual mode calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating dual mode detection catalogue '
                 '**********'),
                 log_file)
    print("")

    detection_name = f'sex_{field}_detection_withIDs.fits'
    detection_file = os.path.join(dual_id_path, detection_name)

    save_name = f'{field}_detection_dual.fits'
    save_file = os.path.join(catalogs_path_dual, save_name)

    if not os.path.exists(save_file):

        ut.sexcatalog_detection(detection_file   = detection_file,
                                save_file        = save_file,
                                calibration_flag = conf['calibration_flag'],
                                field            = field,
                              extinction_maps_path=conf['extinction_maps_path'],
                          extinction_correction = conf['extinction_correction'],
                                calibration_name=conf['calibration_name'])

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_dual:
    generate_detection_catalogue()


# ***************************************************
#    Make PSF filter catalogs
# ***************************************************


def generate_psf_filter_catalogues():

    """
    Generates PSF calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating PSF calibrated filter catalogues '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        catalog_name = f'{field}_{filt}_psf_withIDs.csv'
        catalog_file = os.path.join(psf_id_path, catalog_name)

        zp_name = f'{field}_final.zp'
        zp_file = os.path.join(calibration_path, zp_name)

        save_name = f'{field}_{filt}_psf.fits'
        save_file = os.path.join(catalogs_path_psf, save_name)

        if not os.path.exists(save_file):

            ut.psfcatalog_apply_calibration(catalog_file = catalog_file,
                                            zp_file      = zp_file,
                                            save_file    = save_file,
                                            filter_name  = filt,
                                            field        = field,
                                            inst_mag_zp  = conf['inst_zp'],
                                      calibration_flag=conf['calibration_flag'],
                                      calibration_name=conf['calibration_name'],
                              extinction_maps_path=conf['extinction_maps_path'],
                            extinction_correction=conf['extinction_correction'])

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_psf:
    generate_psf_filter_catalogues()


# ***************************************************
#    Make single mode 32-aper filter catalogs
# ***************************************************

def generate_singlemode_32aper_filter_catalogues():

    """
    Generates single mode 32-aper calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating single mode 32-aper calibrated filter catalogues '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        catalog_name = f'sex_{field}_{filt}_single_withIDs.fits'
        catalog_file = os.path.join(single_id_path, catalog_name)

        zp_name = f'{field}_final.zp'
        zp_file = os.path.join(calibration_path, zp_name)

        save_name = f'{field}_{filt}_single_aper.fits'
        save_file = os.path.join(catalogs_path_single, save_name)

        if not os.path.exists(save_file):

            ut.sexcatalog_apply_calibration_aper(catalog_file = catalog_file,
                                            zp_file      = zp_file,
                                            save_file    = save_file,
                                            filter_name  = filt,
                                            sex_mag_zp   = conf['inst_zp'],
                                            mode         = 'single',
                                            field        = field,
                                        drname=conf["data_release_name"])

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_single:
    if conf["create_aper_catalog"]:
        generate_singlemode_32aper_filter_catalogues()


# ***************************************************
#    Make dual mode 32-aper filter catalogs
# ***************************************************

def generate_dualmode_32aper_filter_catalogues():

    """
    Generates dual mode 32-aper calibrated filter catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Generating dual mode 32-aper calibrated filter catalogues '
                 '**********'),
                 log_file)
    print("")

    for filt in conf['filters']:

        catalog_name = f'sex_{field}_{filt}_dual_withIDs.fits'
        catalog_file = os.path.join(dual_id_path, catalog_name)

        zp_name = f'{field}_final.zp'
        zp_file = os.path.join(calibration_path, zp_name)

        save_name = f'{field}_{filt}_dual_aper.fits'
        save_file = os.path.join(catalogs_path_dual, save_name)

        if not os.path.exists(save_file):

            ut.sexcatalog_apply_calibration_aper(catalog_file = catalog_file,
                                            zp_file      = zp_file,
                                            save_file    = save_file,
                                            filter_name  = filt,
                                            sex_mag_zp   = conf['inst_zp'],
                                            mode         = 'dual',
                                            field=field,
                                            drname=conf["data_release_name"])

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_dual:
    if conf["create_aper_catalog"]:
        generate_dualmode_32aper_filter_catalogues()


# ***************************************************
#    Extract VAC features from detection
# ***************************************************

def extract_vac_features_from_dual_mode():

    """
    Extracts VACs features from detection catalog
    """

    print("")
    ut.printlog(('********** '
                 'Extracting VAC features from detection catalog '
                 '**********'),
                 log_file)
    print("")

    for filt in ["detection"] + conf['filters']:

        if filt == "detection":
            catalog_name = f'{field}_detection_dual.fits'
            catalog_file = os.path.join(catalogs_path_dual, catalog_name)

            save_name = f'{field}_detection_dual_VAC_features.fits'
            save_file = os.path.join(catalogs_path_dual, save_name)
        else:
            catalog_name = f'{field}_{filt}_dual.fits'
            catalog_file = os.path.join(catalogs_path_dual, catalog_name)

            save_name = f'{field}_{filt}_dual_VAC_features.fits'
            save_file = os.path.join(catalogs_path_dual, save_name)

        future_name = f'{field}_dual_VAC_features.fits'
        future_file = os.path.join(catalogs_path_dual, future_name)

        if not os.path.exists(save_file) and not os.path.exists(future_file):
            ut.extract_vac_features(catalog = catalog_file,
                                    save_file = save_file,
                                    filt = filt)

            ut.printlog(f"Created file {save_file}", log_file)

        else:
            ut.printlog(f"Catalog {save_name} already exists", log_file)


if has_photometry_dual:
    extract_vac_features_from_dual_mode()


# ***************************************************
#    Combine VAC features
# ***************************************************

def combine_vac_features():
    print("")
    ut.printlog(('********** '
                 'Combining VAC features '
                 '**********'),
                log_file)
    print("")

    save_name = f'{field}_dual_VAC_features.fits'
    save_file = os.path.join(catalogs_path_dual, save_name)

    if not os.path.exists(save_file):

        cmd  = f"java -jar {conf['path_to_stilts']} tmatchn "
        cmd += f"nin={1+len(conf['filters'])} matcher=exact "
        cmd += f"multimode=group "

        catalog_name = f'{field}_detection_dual_VAC_features.fits'
        catalog = os.path.join(catalogs_path_dual, catalog_name)

        valuesj = f"PHOT_ID_dual"

        cmd += f"in1={catalog} ifmt1=fits values1={valuesj} "
        cmd += f"join1=always "

        j = 2
        for filt in conf['filters']:
            catalog_name = f'{field}_{filt}_dual_VAC_features.fits'
            catalog = os.path.join(catalogs_path_dual, catalog_name)

            valuesj = f"PHOT_ID_dual"

            cmd += f"in{j}={catalog} ifmt{j}=fits values{j}={valuesj} "
            cmd += f"join{j}=always "
            j += 1

        cmd += f"out={save_file} ofmt=fits"

        ut.printlog(cmd, log_file)
        os.system(cmd)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


if 'photometry_dual' in conf['run_steps']:
    combine_vac_features()


# ***************************************************
#    Remove temporary VAC files
# ***************************************************

def remove_temporary_vac_features():
    print("")
    ut.printlog(('********** '
                 'Removing temporary VAC features '
                 '**********'),
                log_file)
    print("")

    vac_name = f'{field}_dual_VAC_features.fits'
    vac_file = os.path.join(catalogs_path_dual, vac_name)

    if os.path.exists(vac_file):

        catalog_name = f'{field}_detection_dual_VAC_features.fits'
        catalog = os.path.join(catalogs_path_dual, catalog_name)

        cmd = f"rm {catalog} "
        ut.printlog(cmd, log_file)
        os.system(cmd)

        for filt in conf['filters']:
            catalog_name = f'{field}_{filt}_dual_VAC_features.fits'
            catalog = os.path.join(catalogs_path_dual, catalog_name)

            cmd = f"rm {catalog} "
            ut.printlog(cmd, log_file)
            os.system(cmd)


if 'photometry_dual' in conf['run_steps']:
    remove_temporary_vac_features()



# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                           calibration_gaiascale.py
#               Runs the internal calibration step of the pipeline
# ******************************************************************************

"""
Obtains the gaia scale calibration zero-points by fitting models to s-plus
previously calibrated catalog and comparing predicted and observed Gaia
magnitudes

The S-PLUS field is given as the first command line argument. Configurations
are set in the config file, given as the second command line argument.

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------

crossmatch_splus_gaia()
fit_sed_to_splus()
get_gaia_gaiascale_zero_points()
get_splus_gaiascale_zero_points()
apply_gaiascale_zero_points()
plot_gaiascale_zp()

--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

Assumes that crossmatch.py has already been run for this field.

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 calibration_external.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys

steps_path = os.path.split(__file__)[0]
pipeline_path = os.path.split(steps_path)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

################################################################################
# Get directories

field_path       = os.path.join(conf['run_path'], field)
crossmatch_path  = os.path.join(field_path, 'Crossmatch')

suffix = ut.calibration_suffix(conf)
calibration_path  = os.path.join(field_path, f'Calibration_{suffix}')

external_cal_path = os.path.join(calibration_path, 'external')
stloc_cal_path = os.path.join(calibration_path, 'stlocus')
internal_cal_path = os.path.join(calibration_path, 'internal')

gaiascale_cal_path = os.path.join(calibration_path, 'gaiascale')
gaiascale_plot_path = os.path.join(gaiascale_cal_path, 'plots')

log_path = os.path.join(calibration_path, 'logs')


################################################################################
# Initiate log file

ut.makedir(calibration_path)
ut.makedir(gaiascale_cal_path)
ut.makedir(gaiascale_plot_path)
ut.makedir(log_path)

log_file_name = os.path.join(log_path, 'calibration_gaiascale.log')
log_file_name = ut.gen_logfile_name(log_file_name)
log_file = os.path.join(calibration_path, log_file_name)

with open(log_file, "w") as log:
    log.write("")

################################################################################
# Begin script


# ***************************************************
#    Crossmatch S-PLUS and Gaia Catalog
# ***************************************************

def crossmatch_splus_gaia():

    """
    Crossmatch S-PLUS and Gaia catalogues
    """

    print("")
    ut.printlog(('********** '
                 'Crossmatching S-PLUS (calibrated) and Gaia'
                 ' **********'),
                log_file)
    print("")

    # Get latest S-PLUS calibrated catalog for the field
    if 'calibration_internal' in conf['run_steps']:
        splus_name = f"{field}_mag_int.cat"
        splus_file = os.path.join(internal_cal_path, splus_name)

    else:
        if 'calibration_stloc' in conf['run_steps']:
            splus_name = f"{field}_mag_stlocus.cat"
            splus_file = os.path.join(stloc_cal_path, splus_name)

        else:
            splus_name = f"{field}_mag_ext.cat"
            splus_file = os.path.join(external_cal_path, splus_name)

    # Get gaia catalog file for the field
    gaia_file = os.path.join(crossmatch_path, f'{field}_GAIADR2.fits')

    save_file = os.path.join(gaiascale_cal_path, 'splus_gaia.fits')

    if not os.path.exists(save_file):
        cmd  = f"java -jar {conf['path_to_stilts']} tmatch2 "

        cmd += f"in1={splus_file} ifmt1=ascii "
        cmd += f"values1='RAJ2000 DEJ2000' "

        cmd += f"in2={gaia_file} ifmt2=fits "
        cmd += f"values2='GAIADR2_RAJ2000 GAIADR2_DEJ2000' "

        cmd += f"join=1and2 find=best matcher=sky params=3 "

        cmd += f"ofmt=fits out={save_file} "

        ut.printlog(cmd, log_file)
        os.system(cmd)

        ut.printlog(f"Created catalog {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_file} already exists", log_file)


crossmatch_splus_gaia()


# ***************************************************
#    Apply extinction correction to Gaia
# ***************************************************

def correct_extinction_gaia_magnitudes():
    """
    Applying extinction correction to Gaia magnitudes
    """

    print("")
    ut.printlog(('********** '
                 'Applying extinction correction to Gaia magnitudes '
                 '**********'),
                 log_file)
    print("")

    ###############################
    # Get crossmatched catalog file
    cmatch_file = os.path.join(gaiascale_cal_path, 'splus_gaia.fits')

    ##################
    # Apply correction
    correction = conf['extinction_correction']
    ebv_map_path = conf['extinction_maps_path']

    save_file = cmatch_file.replace(".fits", "_ebvcorr.fits")

    # S-PLUS magnitudes are already extinction corrected, so apply only to Gaia
    # reference http://svo2.cab.inta-csic.es/theory/fps/index.php?mode=browse&gname=GAIA&gname2=GAIA2r&asttype=
    filters_Alambda_Av = {'GAIA_BP': 1.13,
                          'GAIA_G': 0.939,
                          'GAIA_RP': 0.659}

    if not os.path.exists(save_file):
        ut.correct_extinction(catalog = cmatch_file,
                              save_file = save_file,
                              correction = correction,
                              ebv_maps_path = ebv_map_path,
                              filters_Alambda_Av = filters_Alambda_Av)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Extinction corrected catalog already exists", log_file)


if conf['extinction_correction'].lower() != 'none':
    correct_extinction_gaia_magnitudes()


# ***************************************************
#    Convert cmatch catalog to ascii
# ***************************************************

def convert_gaia_xmatch_2ascii():
    """
    Convert cmatch catalog to ascii
    """

    print("")
    ut.printlog(('********** '
                 'Converting Gaia crossmatch to ascii '
                 '**********'),
                 log_file)
    print("")

    if conf['extinction_correction'].lower() != 'none':
        splus_gaia_catalog = f"splus_gaia_ebvcorr.fits"
    else:
        splus_gaia_catalog = f"splus_gaia.fits"

    splus_gaia_file = os.path.join(gaiascale_cal_path, splus_gaia_catalog)

    save_file = splus_gaia_file.replace(".fits", ".cat")

    if not os.path.exists(save_file):

        cmd  = f"java -jar {conf['path_to_stilts']} tcopy "

        cmd += f"ifmt=fits in={splus_gaia_file} "
        cmd += f"ofmt=ascii out={save_file}"

        ut.printlog(cmd, log_file)
        os.system(cmd)

    else:
        ut.printlog(f"File {save_file} already exists", log_file)


convert_gaia_xmatch_2ascii()

# ***************************************************
#    Fit SEDs to splus magnitudes
# ***************************************************


def fit_sed_to_splus():

    """
    Fits SEDs to the splus catalog magnitudes to predict the gaia magnitudes
    """

    print("")
    ut.printlog(('********** '
                 'Fitting SEDs to splus '
                 '**********'),
                 log_file)
    print("")

    models_file = conf['path_to_models']

    if conf['extinction_correction'].lower() != 'none':
        splus_gaia_catalog = f"splus_gaia_ebvcorr.cat"
    else:
        splus_gaia_catalog = f"splus_gaia.cat"

    splus_gaia_file = os.path.join(gaiascale_cal_path, splus_gaia_catalog)

    bayesian_fitting = conf['model_fitting_bayesian']
    ebv_mode = conf['model_fitting_ebv_cut']

    ref_mag_cols  = conf['gaiascale_sed_fit']
    pred_mag_cols = conf['gaiascale_sed_pred'] + conf['external_sed_fit']

    save_name = f"{field}_gaiascale_sed_fit.cat"
    save_file = os.path.join(gaiascale_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.sed_fitting(models        = models_file,
                       catalog       = splus_gaia_file,
                       save_file     = save_file,
                       ref_mag_cols  = ref_mag_cols,
                       pred_mag_cols = pred_mag_cols,
                       bayesian      = bayesian_fitting,
                       ebv_mode      = ebv_mode)

        ut.printlog(f"Generated SED fit catalog {save_file}", log_file)
    else:
        ut.printlog(f"SED fit catalog {save_name} already exists", log_file)


fit_sed_to_splus()


# ***************************************************
#    Get external calibration zero points
# ***************************************************

def get_gaia_gaiascale_zero_points():

    """
    Obtains the gaia scale calibration zero-points in gaia magnitudes
    """

    print("")
    ut.printlog(('********** '
                 'Getting gaia scale calibration for gaia magnitudes '
                 '**********'),
                 log_file)
    print("")

    catalog_name = f"{field}_gaiascale_sed_fit.cat"
    catalog_file = os.path.join(gaiascale_cal_path, catalog_name)

    save_name = f"{field}_gaiascale_gaia.zp"
    save_file = os.path.join(gaiascale_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.zp_estimate(catalog = catalog_file,
                       save_file = save_file,
                       filters_list = conf['gaiascale_sed_pred'],
                       mag_cut   = conf['gaia_zp_fitting_mag_cut'])

        ut.printlog(f"Created zp file {save_file}", log_file)

    else:
        ut.printlog(f"ZP file {save_name} already exists", log_file)


get_gaia_gaiascale_zero_points()


# ***************************************************
#    Get splus gaiascale zero points
# ***************************************************

def get_splus_gaiascale_zero_points():

    """
    Obtains the gaia scale calibration zero-points for the S-PLUS filters
    """

    print("")
    ut.printlog(('********** '
                 'Getting gaia scale calibration for S-PLUS magnitudes '
                 '**********'),
                 log_file)
    print("")

    gaia_zp_name = f"{field}_gaiascale_gaia.zp"
    gaia_zp_file = os.path.join(gaiascale_cal_path, gaia_zp_name)

    filters_list = []
    for filt in conf['filters']:
        filters_list.append(f'SPLUS_{filt}')

    save_name = f"{field}_gaiascale_splus_tmp.zp"
    save_file = os.path.join(gaiascale_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.zp_gaiascale(gaia_zp_file = gaia_zp_file,
                        save_file    = save_file,
                        filters_list = filters_list)

        ut.printlog(f"Created zp file {save_file}", log_file)

    else:
        ut.printlog(f"ZP file {save_name} already exists", log_file)


get_splus_gaiascale_zero_points()


# ***************************************************
#    Add offset to S-PLUS reference calibration
# ***************************************************

def add_offset_to_splus_reference_calibration():
    """
    Adds offset to S-PLUS reference calibration
    """

    if conf['offset_include_after'].lower() == "gaiascale":
        print("")
        ut.printlog(('********** '
                     'Adding offset to S-PLUS reference calibration '
                     '**********'),
                    log_file)
        print("")

    gaiascale_zp_name = f"{field}_gaiascale_splus_tmp.zp"
    gaiascale_zp_file = os.path.join(gaiascale_cal_path, gaiascale_zp_name)

    save_name = f"{field}_gaiascale_splus.zp"
    save_file = os.path.join(gaiascale_cal_path, save_name)

    if not os.path.exists(save_file):

        if conf['offset_include_after'].lower() == "gaiascale":
            offset_zp_name = f'offset_refcalib.zp'
            offset_zp_file = os.path.join(gaiascale_cal_path, offset_zp_name)

            cmd = f"cp {conf['offset_to_splus_refcalib']} {offset_zp_file}"
            ut.printlog(cmd, log_file)
            os.system(cmd)

            zp_file_list = [gaiascale_zp_file, offset_zp_file]

            filters = []
            for filt in conf['filters']:
                filters.append(f"SPLUS_{filt}")

            # Combine zero-points
            ut.zp_add(zp_file_list=zp_file_list,
                      save_file=save_file,
                      filters=filters,
                      inst_zp=0)

        else:
            cmd = f"cp {gaiascale_zp_file} {save_file}"
            os.system(cmd)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"File {save_name} already exists", log_file)


add_offset_to_splus_reference_calibration()

# ***************************************************
#    Apply Zero Points
# ***************************************************

def apply_gaiascale_zero_points():

    """
    Applies gaiascale zero-points to latest catalog
    """

    print("")
    ut.printlog(('********** '
                 'Applying gaia scale zero-points to latest catalog '
                 '**********'),
                 log_file)
    print("")

    catalog_name = f"{field}_gaiascale_sed_fit.cat"
    catalog_file = os.path.join(gaiascale_cal_path, catalog_name)

    zp_name = f"{field}_gaiascale_splus.zp"
    zp_file = os.path.join(gaiascale_cal_path, zp_name)

    save_name = f"{field}_mag_gaiascale.cat"
    save_file = os.path.join(gaiascale_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.zp_apply(catalog   = catalog_file,
                    save_file = save_file,
                    zp_file   = zp_file)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


apply_gaiascale_zero_points()

# ***************************************************
#          Plot internal ZPs fitting
# ***************************************************


def plot_gaiascale_zp():

    """
    Makes diagnostic plots of the gaia scale zp fitting process
    """

    print("")
    ut.printlog(('********** '
                 'Plotting gaia scale zp-fitting process '
                 '**********'),
                 log_file)
    print("")

    sed_fit_name = f"{field}_gaiascale_sed_fit.cat"
    sed_fit_file = os.path.join(gaiascale_cal_path, sed_fit_name)

    zp_name = f"{field}_gaiascale_gaia.zp"
    zp_file = os.path.join(gaiascale_cal_path, zp_name)

    for filt in conf['gaiascale_sed_pred']:
        save_name = f"{field}_{filt}_gaiascale.png"
        save_file = os.path.join(gaiascale_plot_path, save_name)

        if not os.path.exists(save_file):
            ut.plot_zp_fitting(sed_fit_file = sed_fit_file,
                               zp_file      = zp_file,
                               save_file    = save_file,
                               filt         = filt,
                               mag_cut      = conf['gaia_zp_fitting_mag_cut'])

            ut.printlog(f"Saved plot {save_file}", log_file)

        else:
            ut.printlog(f"Plot {save_name} already made", log_file)


plot_gaiascale_zp()


# -*- coding: utf-8 -*-

# ******************************************************************************
#                          S-PLUS CALIBRATION PIPELINE
#                           calibration_diagnostic.py
#               Runs the diagnostic calibration step of the pipeline
# ******************************************************************************

"""
Predicts the reference catalog magnitudes by fitting models to s-plus
previously calibrated catalog. This can be used to check the accuracy of
the S-PLUS calibration after comparing to the real values of the magnitudes
in the reference catalog.

The S-PLUS field is given as the first command line argument. Configurations
are set in the config file, given as the second command line argument.

--------------------------------------------------------------------------------
   FUNCTIONS:
--------------------------------------------------------------------------------

fit_sed_to_splus_diagnostic()

--------------------------------------------------------------------------------
   COMMENTS:
--------------------------------------------------------------------------------
Ideally this script should only be run through the pipeline.py script.

Assumes that crossmatch.py has already been run for this field.

--------------------------------------------------------------------------------
   USAGE:
--------------------------------------------------------------------------------
$python3 calibration_external.py *field_name* *config_file*

----------------
"""

################################################################################
# Import external packages

import os
import sys

steps_path = os.path.split(__file__)[0]
pipeline_path = os.path.split(steps_path)[0]
spluscalib_path = os.path.split(pipeline_path)[0]

sys.path.append(spluscalib_path)

################################################################################
# Import spluscalib packages

from spluscalib import utils as ut

################################################################################
# Read parameters

field     = sys.argv[1]
conf_file = sys.argv[2]

conf = ut.pipeline_conf(conf_file)

################################################################################
# Get directories

field_path       = os.path.join(conf['run_path'], field)

suffix = ut.calibration_suffix(conf)
calibration_path  = os.path.join(field_path, f'Calibration_{suffix}')
crossmatch_path   = os.path.join(field_path, f'Crossmatch')

external_cal_path  = os.path.join(calibration_path, 'external')
stloc_cal_path     = os.path.join(calibration_path, 'stlocus')
internal_cal_path  = os.path.join(calibration_path, 'internal')
gaiascale_cal_path = os.path.join(calibration_path, 'gaiascale')

diagnostic_cal_path = os.path.join(calibration_path, 'diagnostic')

log_path = os.path.join(calibration_path, 'logs')


################################################################################
# Initiate log file

ut.makedir(diagnostic_cal_path)
ut.makedir(log_path)

log_file_name = os.path.join(log_path, 'calibration_diagnostic.log')
log_file_name = ut.gen_logfile_name(log_file_name)
log_file = os.path.join(calibration_path, log_file_name)

with open(log_file, "w") as log:
    log.write("")


################################################################################
# Begin script

# ***************************************************
#    Prepare catalog with reference magnitudes
# ***************************************************

def prepare_diagnostic_input():
    """
    Prepares diagnostic input catalog
    """

    print("")
    ut.printlog(('********** '
                 'Preparing diagnostic input catalog '
                 '**********'),
                 log_file)
    print("")

    if 'calibration_gaiascale' in conf['run_steps']:
        cat_name = f"{field}_mag_gaiascale.cat"
        cat_file = os.path.join(gaiascale_cal_path, cat_name)
    else:
        if 'calibration_internal' in conf['run_steps']:
            cat_name = f"{field}_mag_int.cat"
            cat_file = os.path.join(internal_cal_path, cat_name)
        else:
            cat_name = f"{field}_mag_ext.cat"
            cat_file = os.path.join(internal_cal_path, cat_name)

    select_columns = ['RAJ2000', 'DEJ2000', 'X', 'Y']

    for filt in conf['diagnostic_sed_fit']:
        select_columns.append(f"{filt}")
        select_columns.append(f"{filt}_err")

    for filt in conf['external_sed_fit']:
        select_columns.append(f"{filt}")
        select_columns.append(f"{filt}_err")

    save_name = f'{field}_diagnostic_input.cat'
    save_file = os.path.join(diagnostic_cal_path, save_name)

    if not os.path.exists(save_file):

        ut.select_columns(catalog        = cat_file,
                          save_file      = save_file,
                          select_columns = select_columns)

        ut.printlog(f"Created catalog {save_file}", log_file)

    else:
        ut.printlog(f"File {save_file} already exists.", log_file)


prepare_diagnostic_input()


# ***************************************************
#    Predict reference magnitudes
# ***************************************************

def fit_sed_to_splus_diagnostic():

    """
    Fits SEDs to the splus catalog magnitudes to predict the gaia magnitudes
    """

    print("")
    ut.printlog(('********** '
                 'Fitting SEDs to splus '
                 '**********'),
                 log_file)
    print("")

    models_file = conf['path_to_models']

    cat_name = f'{field}_diagnostic_input.cat'
    cat_file = os.path.join(diagnostic_cal_path, cat_name)

    msg = f"Fitting models to file {cat_file}"
    ut.printlog(msg, log_file)

    bayesian_fitting = conf['model_fitting_bayesian']
    ebv_mode = conf['model_fitting_ebv_cut']

    ref_mag_cols  = conf['diagnostic_sed_fit']
    pred_mag_cols = conf['external_sed_fit']

    if conf['extinction_correction'].lower() != 'none':
        save_name = 'diagnostic_mag_ebvcorr.cat'
    else:
        save_name = 'diagnostic_mag.cat'
    save_file = os.path.join(diagnostic_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.sed_fitting(models        = models_file,
                       catalog       = cat_file,
                       save_file     = save_file,
                       ref_mag_cols  = ref_mag_cols,
                       pred_mag_cols = pred_mag_cols,
                       bayesian      = bayesian_fitting,
                       ebv_mode      = ebv_mode)

        ut.printlog(f"Generated SED fit catalog {save_file}", log_file)
    else:
        ut.printlog(f"SED fit catalog {save_name} already exists", log_file)


fit_sed_to_splus_diagnostic()


# ***************************************************
#    Convert to fits
# ***************************************************

def convert_diagnostic_2fits():
    """
    Converts diagnostic catalog to fits
    """

    print("")
    ut.printlog(('********** '
                 'Converting diagnostic catalog to fits '
                 '**********'),
                 log_file)
    print("")

    if conf['extinction_correction'].lower() != 'none':
        cat_name = 'diagnostic_mag_ebvcorr.cat'
    else:
        cat_name = 'diagnostic_mag.cat'

    cat_file = os.path.join(diagnostic_cal_path, cat_name)

    save_file = cat_file.replace(".cat", ".fits")

    if not os.path.exists(save_file):

        cmd  = f"java -jar {conf['path_to_stilts']} tcopy "

        cmd += f"ifmt=ascii in={cat_file} "
        cmd += f"ofmt=fits out={save_file}"

        ut.printlog(cmd, log_file)
        os.system(cmd)

    else:
        ut.printlog(f"File {save_file} already exists", log_file)


convert_diagnostic_2fits()


# ***************************************************
#    Remove extinction correction
# ***************************************************

def remove_extinction_correction():
    """
    Removes extinction correction from reference magnitudes (also from models)
    """

    print("")
    ut.printlog(('********** '
                 'Removing exctinction correction '
                 '**********'),
                 log_file)
    print("")

    ###############################
    # Get crossmatched catalog file
    cat_name = 'diagnostic_mag_ebvcorr.fits'
    cat_file = os.path.join(diagnostic_cal_path, cat_name)

    ##################
    # Apply correction
    correction = conf['extinction_correction']
    ebv_map_path = conf['extinction_maps_path']

    save_name = f'{field}_diagnostic_mag.fits'
    save_file = os.path.join(diagnostic_cal_path, save_name)

    if not os.path.exists(save_file):
        ut.correct_extinction(catalog = cat_file,
                              save_file = save_file,
                              correction = correction,
                              ebv_maps_path = ebv_map_path,
                              reverse = True,
                              include_mod = True)

        ut.printlog(f"Created file {save_file}", log_file)

    else:
        ut.printlog(f"Removed extinction correction catalog already exists",
                    log_file)


if conf['extinction_correction'].lower() != 'none':
    remove_extinction_correction()


# ******************************************************************************
#    !!!!!!!!!!!!!!!!!!!!!!!!!!! OBSOLETE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# ******************************************************************************

# ***************************************************
#    Crossmatch with external catalog
# ***************************************************

def crossmatch_with_reference():

    """
    Crossmatches diagnostic predictions with extinction corrected reference
    """

    print("")
    ut.printlog(('********** '
                 'Crossmatching diagnostic predictions with extinction '
                 'corrected reference'
                 '**********'),
                 log_file)
    print("")

    model_mag_name = f"{field}_diagnostic_sed_fit.cat"
    model_mag_file = os.path.join(diagnostic_cal_path, model_mag_name)

    # Here I'm taking the file used in the external calibration step, since
    # it has the EB-V corrected reference magnitudes
    ref_mag_name = f"{field}_mag_inst.cat"
    ref_mag_file = os.path.join(external_cal_path, ref_mag_name)

    save_name = 'diagnostic_mag_temp.cat'
    save_file = os.path.join(diagnostic_cal_path, save_name)

    if not os.path.exists(save_file):

        cmd  = f"java -jar {conf['path_to_stilts']} tmatch2 "

        cmd += f"in1={ref_mag_file} ifmt1=ascii values1='RAJ2000 DEJ2000' "
        cmd += f"in2={model_mag_file} ifmt2=ascii values2='RAJ2000 DEJ2000' "

        cmd += f"out={save_file} ofmt=ascii "

        cmd += f"join=1and2 matcher=sky params=1"

        ut.printlog(cmd, log_file)
        os.system(cmd)

        ut.printlog(f"Created catalog {save_file}", log_file)

    else:
        ut.printlog(f"Catalog {save_name} already exists", log_file)


#crossmatch_with_reference()


# ***************************************************
#    Remove unwanted columns
# ***************************************************

def remove_unwanted_cols():
    """
    Remove unwanted columns from diagnostic file
    """

    print("")
    ut.printlog(('********** '
                 'Removing unwanted columns from diagnostic file '
                 '**********'),
                log_file)
    print("")

    cat_name = 'diagnostic_mag_temp.cat'
    cat_file = os.path.join(diagnostic_cal_path, cat_name)

    if conf['extinction_correction'].lower() != 'none':
        save_name = 'diagnostic_mag_ebvcorr.cat'
    else:
        save_name = 'diagnostic_mag.cat'

    save_file = os.path.join(diagnostic_cal_path, save_name)

    select_columns = ['RAJ2000_1', 'DEJ2000_1', 'X', 'Y', 'model_id', 'Teff',
                      'logg', 'FeH', 'aFe', 'EB_V', 'chi2', 'model_mag_shift']

    for filt in conf['external_sed_fit']:
        select_columns.append(f'{filt}')
        select_columns.append(f'{filt}_err')
        select_columns.append(f'{filt}_mod')

    rename_columns = {'RAJ2000_1': 'RAJ2000', 'DEJ2000_1': 'DEJ2000',
                      'EB_V': 'EB_V_model'}

    if not os.path.exists(save_file):

        ut.select_columns(catalog        = cat_file,
                          save_file      = save_file,
                          select_columns = select_columns,
                          rename_columns = rename_columns)

        ut.printlog(f"Created catalog {save_file}", log_file)

        ut.printlog(f"Removing temporary catalog", log_file)

        cmd = f"rm {cat_file}"
        ut.printlog(cmd, log_file)
        os.system(cmd)

    else:
        ut.printlog(f"File {save_file} already exists.", log_file)


#remove_unwanted_cols()
